// ignore_for_file: non_constant_identifier_names

import 'dart:math';

import 'package:collection/collection.dart';
import 'package:general_lib/extension/extension.dart';
import 'package:general_lib/scheme/captcha_word.dart';

class CaptchaGeneralLib {
  List<Map<String, String>> animalEmoji = [
    {
      "emoji": "🦊️",
      "name": "fox",
    },
    {
      "emoji": "🐒️",
      "name": "monkey",
    },
    {
      "emoji": "🦍️",
      "name": "gorilla",
    },
    {
      "emoji": "🐯️",
      "name": "tiger",
    },
    {
      "emoji": "🦖️",
      "name": "dinosaur",
    },
    {
      "emoji": "🐝️",
      "name": "bee",
    },
    {
      "emoji": "🐘️",
      "name": "elephant",
    },
    {
      "emoji": "🐷️",
      "name": "pig",
    },
    {
      "emoji": "🐍️",
      "name": "snake",
    },
    {
      "emoji": "🦂️",
      "name": "scorpion",
    }
  ];

  List<String> default_words = [
    "guru",
    "dokter",
    "bekerja",
    "belajar",
    "bermain",
    "game",
    "makanan",
    "duduk",
    "air",
    "minuman",
    "mangga",
    "bakso",
  ];

  CaptchaGeneralLib();

  CaptchaResult animalCaptcha({
    List<Map>? emojis,
    int captchaLength = 5,
    Duration timeOut = const Duration(minutes: 1),
  }) {
    emojis ??= animalEmoji;
    if (captchaLength > emojis.length) {
      throw "captchaLength melebihi emojis length";
    }
    Map jsonKey = emojis[Random().nextInt(emojis.length)];
    List<Map> result = [jsonKey];
    DateTime dateTime = DateTime.now().add(timeOut);

    while (true) {
      if (dateTime.isBefore(DateTime.now())) {}

      if (result.length == captchaLength) {
        return CaptchaResult(
          answer: jsonKey,
          captchas: result,
        );
      }
      Map randomEmoji = emojis[Random().nextInt(emojis.length)];

      if (!DeepCollectionEquality().equals(randomEmoji, jsonKey)) {
        bool isNotFoundSame = true;
        for (var index = 0; index < result.length; index++) {
          if (DeepCollectionEquality().equals(randomEmoji, result[index])) {
            isNotFoundSame = false;
          }
        }
        if (isNotFoundSame) {
          result.add(randomEmoji);
          result.shuffle();
        }
      }
    }
  }

  T randomByDatas<T>(List<T> datas) {
    return datas[Random().nextInt(datas.length)];
  }

  CaptchaWord words({
    List<String>? words,
    int captchaLength = 5,
    Duration timeOut = const Duration(minutes: 1),
  }) {
    words ??= default_words;
    if (captchaLength > words.length) {
      throw Exception("captchaLength melebihi words length");
    }
    String word = randomByDatas(words);
    List<String> results = [word];
    DateTime dateTime = DateTime.now().add(timeOut);

    int total_answer = captchaLength ~/ 4;

    if (total_answer < 5) {
      throw Exception("Captcha word total harus lebih 5");
    }

    while (true) {
      if (dateTime.isExpired()) {
        throw Exception("Captcha word time expired");
      }

      if (results.length == captchaLength) {
        List<int> answers = [];

        while (true) {
          if (dateTime.isExpired()) {
            throw Exception("Captcha word time expired");
          }
          int random_data = Random().nextInt(captchaLength);
          if (answers.length == total_answer) {
            break;
          }
          if (answers.contains(random_data)) {
          } else {
            answers.add(random_data);
          }
        }
        return CaptchaWord.create(
          special_type: "captchaWord",
          answer: answers,
          words: results,
        );
      }
      if (results.contains(word)) {
        word = randomByDatas(words);
      } else {
        results.add(word);
      }
    }
  }
}

class CaptchaResult {
  final Map answer;
  final List captchas;
  CaptchaResult({
    required this.answer,
    required this.captchas,
  });
  Map toJson() {
    return {
      "answer": answer,
      "captchas": captchas,
    };
  }
}
