// ignore_for_file: non_constant_identifier_names, unnecessary_brace_in_string_interps

import 'dart:convert';

import 'package:general_lib/dart/dart.dart';
import "package:path/path.dart" as path;
import 'package:universal_io/io.dart';

class OperatingSystem {
  OperatingSystem();

  static Directory get app_directory {
    String? configDir;
    if (Platform.isLinux) {
      configDir = Platform.environment['XDG_CONFIG_HOME'] ??
          path.join(Platform.environment['HOME']!, '.config');
    } else if (Platform.isWindows) {
      configDir = Platform.environment['APPDATA']!;
    } else if (Platform.isMacOS) {
      configDir = path.join(
          Platform.environment['HOME']!, 'Library', 'Application Support');
    } else {
      configDir = path.join(Platform.environment['HOME'] ?? '', '.config');
    }

    return Directory(configDir);
  }

  static String? getEnvironment(String key, [String? defaultValue]) {
    return Platform.environment[key] ?? defaultValue;
  }

  static Future<bool> setEnvironment(String key, String value) async {
    if (dart.isWeb) {
      return false;
    }
    if (dart.isLinux || dart.isMacOS) {
      File file = File(path.join(
          Directory(Platform.environment['XDG_CONFIG_HOME'] ??
                  path.join(Platform.environment['HOME']!))
              .path,
          ".bashrc"));
      if (!file.existsSync()) {
        return false;
      }
      String readFile = await file.readAsString();
      List<String> bashrcs = readFile.split("\n");

      RegExp regExp = RegExp(
          "export([ ]+)${key}=${RegExp.escape(json.encode(value))}",
          caseSensitive: false);
      bool is_not_found_path = true;
      for (var i = 0; i < bashrcs.length; i++) {
        String bashrc = bashrcs[i];
        if (regExp.hasMatch(bashrc)) {
          is_not_found_path = false;
        }
      }
      if (is_not_found_path) {
        bashrcs.add("export ${key}=${json.encode(value)}");
        await file.writeAsString(bashrcs.join("\n"));
        await Process.run(
          "source",
          [
            file.path,
          ],
          runInShell: true,
        );
        return true;
      }
      return true;
    }
    if (dart.isWindows) {
      await Process.run(
        "setx",
        [
          key,
          value,
        ],
        runInShell: true,
      );

      return true;
    }

    return false;
  }
}
